@@ -3,7 +3,7 @@
 #
 # SoundConverter - GNOME application for converting between audio formats.
 # Copyright 2004 Lars Wirzenius
-# Copyright 2005-2012 Gautier Portet
+# Copyright 2005-2014 Gautier Portet
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,21 +21,23 @@
 
 import os
 from os.path import basename, dirname
+from random import random
 import time
 import sys
+import urllib
+from gettext import gettext as _
+
 import gtk
 import gobject
 import gnome
 import gnomevfs
-import urllib
-from gettext import gettext as _
-
 from gconfstore import GConfStore
-from fileoperations import filename_to_uri, beautify_uri
+from fileoperations import filename_to_uri, beautify_uri, vfs_writable, \
+    vfs_exists
 from fileoperations import unquote_filename, vfs_walk
 from fileoperations import use_gnomevfs
 from gstreamer import ConverterQueue
-from gstreamer import available_elements, TypeFinder, TagReader
+from gstreamer import available_elements, TypeFinder
 from gstreamer import audio_profiles_list, audio_profiles_dict
 from soundfile import SoundFile
 from settings import locale_patterns_dict, custom_patterns, filepattern, settings
@@ -45,6 +47,7 @@ from utils import log, debug
 from messagearea import MessageArea
 from error import show_error
 
+
 # Names of columns in the file list
 MODEL = [ gobject.TYPE_STRING,   # visible filename
           gobject.TYPE_PYOBJECT, # soundfile
@@ -171,7 +174,8 @@ class FileList:
                              mime_id, time):
         widget.stop_emission('drag_data_received')
         if mime_id >= 0 and mime_id < len(self.drop_mime_types):
-            self.add_uris([uri.strip() for uri in selection.data.split('\n')])
+            uris = [filename_to_uri(uri.strip()) for uri in selection.data.split('\n')]
+            self.add_uris(uris)
             context.finish(True, False, time)
 
     def get_files(self):
@@ -447,7 +451,7 @@ class PreferencesDialog(GladeWindow, GConfStore):
         for k in sorted(locale_patterns_dict.values()):
             tip.append(k)
         self.custom_filename.set_tooltip_text('\n'.join(tip))
-        
+
         #self.resample_rate.connect('changed', self._on_resample_rate_changed)
 
     def convert_setting_from_old_version(self):
@@ -474,6 +478,7 @@ class PreferencesDialog(GladeWindow, GConfStore):
 
         uri = filename_to_uri(self.get_string('selected-folder'))
         self.target_folder_chooser.set_uri(uri)
+        self.target_folder_chooser.set_local_only(not use_gnomevfs)
         self.update_selected_folder()
 
         w = self.create_subfolders
@@ -518,7 +523,7 @@ class PreferencesDialog(GladeWindow, GConfStore):
             self.gstprofile.pack_start(cell)
             self.gstprofile.add_attribute(cell,'text',0)
             self.gstprofile.set_active(0)
-            
+
         # check if we can found the stored audio profile
         found_profile = False
         stored_profile = self.get_string('audio-profile')
@@ -530,12 +535,12 @@ class PreferencesDialog(GladeWindow, GConfStore):
                 found_profile = True
         if not found_profile and stored_profile:
             # reset default output
-            log('Cannot find audio profile "%s", resetting to default output.' 
+            log('Cannot find audio profile "%s", resetting to default output.'
                 % stored_profile)
             self.set_string('audio-profile', '')
             self.gstprofile.set_active(0)
             mime_type = self.defaults['output-mime-type']
-            
+
         self.present_mime_types = []
         i = 0
         model = self.output_mime_type.get_model()
@@ -616,7 +621,7 @@ class PreferencesDialog(GladeWindow, GConfStore):
         else:
             self.custom_filename_box.set_sensitive(False)
 
-        
+
         self.resample_toggle.set_active(self.get_int('output-resample'))
 
         cell = gtk.CellRendererText()
@@ -629,7 +634,7 @@ class PreferencesDialog(GladeWindow, GConfStore):
         except ValueError:
             idx = -1
         self.resample_rate.set_active(idx)
-        
+
         self.force_mono.set_active(self.get_int('force-mono'))
 
         self.jobs.set_active(self.get_int('limit-jobs'))
@@ -768,7 +773,11 @@ class PreferencesDialog(GladeWindow, GConfStore):
         if not self.get_int('same-folder-as-input'):
             folder = self.get_string('selected-folder')
             folder = filename_to_uri(folder)
-        return folder + '/' + basename(soundfile.filename)
+
+        while True:
+            filename = folder + '/' + basename(soundfile.filename) + '~' + str(random())[-6:] + '~SC~'
+            if not vfs_exists(filename):
+                return filename
 
     def process_custom_pattern(self, pattern):
         for k in custom_patterns:
@@ -789,14 +798,14 @@ class PreferencesDialog(GladeWindow, GConfStore):
 
         self.sensitive_widgets['jobs_spinbutton'].set_sensitive(
             self.get_int('limit-jobs'))
-        
+
         if self.get_string('output-mime-type') == 'gst-profile':
             self.sensitive_widgets['resample_hbox'].set_sensitive(False)
             self.sensitive_widgets['force_mono'].set_sensitive(False)
         else:
             self.sensitive_widgets['resample_hbox'].set_sensitive(True)
             self.sensitive_widgets['force_mono'].set_sensitive(True)
-        
+
 
     def run(self):
         self.dialog.run()
@@ -825,6 +834,10 @@ class PreferencesDialog(GladeWindow, GConfStore):
         folder = self.target_folder_chooser.get_uri()
         self.target_folder_chooser.hide()
         if ret == gtk.RESPONSE_OK:
+            writable = vfs_writable(folder)
+            if not writable:
+                show_error(_('Cannot write to folder'), _('"%s" folder is not writable.') % folder)
+                return
             if folder:
                 self.set_string('selected-folder', urllib.unquote(folder))
                 self.update_selected_folder()
